@@ -46,4 +46,4 @@ texinfo_documents = [('index',
 
 
 def setup(app):
-    app.add_stylesheet('custom.css')
+    app.add_css_file('custom.css')
