@@ -130,6 +130,7 @@ class RTNL_API(object):
         # bring it up
         ipr.link('set', index=dev, state='up')
     '''
+
     def __init__(self, *argv, **kwarg):
         if 'netns_path' in kwarg:
             self.netns_path = kwarg['netns_path']
@@ -162,9 +163,10 @@ class RTNL_API(object):
                         else:
                             matches.append(False)
                     else:
-                        matches.append(msg.get(key) == match[key] or
-                                       msg.get_attr(KEY) ==
-                                       match[key])
+                        matches.append(
+                            msg.get(key) == match[key]
+                            or msg.get_attr(KEY) == match[key]
+                        )
                 if all(matches):
                     yield msg
 
@@ -182,19 +184,23 @@ class RTNL_API(object):
         # BSD systems have only subset of the API
         #
         if self.uname[0] == 'OpenBSD':
-            methods = (self.get_links,
-                       self.get_addr,
-                       self.get_neighbours,
-                       self.get_routes)
+            methods = (
+                self.get_links,
+                self.get_addr,
+                self.get_neighbours,
+                self.get_routes,
+            )
         else:
-            methods = (self.get_links,
-                       self.get_addr,
-                       self.get_neighbours,
-                       self.get_routes,
-                       self.get_vlans,
-                       partial(self.fdb, 'dump'),
-                       partial(self.get_rules, family=AF_INET),
-                       partial(self.get_rules, family=AF_INET6))
+            methods = (
+                self.get_links,
+                self.get_addr,
+                self.get_neighbours,
+                self.get_routes,
+                self.get_vlans,
+                partial(self.fdb, 'dump'),
+                partial(self.get_rules, family=AF_INET),
+                partial(self.get_rules, family=AF_INET6),
+            )
         for method in methods:
             for msg in method():
                 yield msg
@@ -251,10 +257,7 @@ class RTNL_API(object):
         # maybe place it as mapping into ifinfomsg.py?
         #
         match = kwarg.get('match', None) or kwarg or None
-        return self.link('dump',
-                         family=AF_BRIDGE,
-                         ext_mask=2,
-                         match=match)
+        return self.link('dump', family=AF_BRIDGE, ext_mask=2, match=match)
 
     def get_links(self, *argv, **kwarg):
         '''
@@ -360,17 +363,19 @@ class RTNL_API(object):
             ip.get_rules() # get all the rules for all families
             ip.get_rules(family=AF_INET6)  # get only IPv6 rules
         '''
-        return self.rule((RTM_GETRULE,
-                          NLM_F_REQUEST | NLM_F_ROOT | NLM_F_ATOMIC),
-                         family=family,
-                         match=match or kwarg)
+        return self.rule(
+            (RTM_GETRULE, NLM_F_REQUEST | NLM_F_ROOT | NLM_F_ATOMIC),
+            family=family,
+            match=match or kwarg,
+        )
 
     def get_routes(self, family=255, match=None, **kwarg):
         '''
         Get all routes. You can specify the table. There
-        are 255 routing classes (tables), and the kernel
+        are up to 4294967295 routing classes (tables), and the kernel
         returns all the routes on each request. So the
-        routine filters routes from full output.
+        routine filters routes from full output. Note the number of
+        tables is increased from 255 in Linux 2.6+.
 
         Example::
 
@@ -386,26 +391,43 @@ class RTNL_API(object):
         '''
         # get a particular route?
         if isinstance(kwarg.get('dst'), basestring):
-            return self.route('get', dst=kwarg['dst'])
+            kwarg = {"dst": kwarg['dst'], "family": family}
+            return self.route('get', **kwarg)
         else:
-            return self.route('dump',
-                              family=family,
-                              match=match or kwarg)
+            return self.route('dump', family=family, match=match or kwarg)
+
     # 8<---------------------------------------------------------------
 
     # 8<---------------------------------------------------------------
+    @staticmethod
+    def open_file(path):
+        '''Open a file (read only) and return its (fd, inode).'''
+        fd = os.open(path, os.O_RDONLY)
+        inode = os.fstat(fd).st_ino
+        return (fd, inode)
+
+    @staticmethod
+    def close_file(fd):
+        '''Close a file that was previously opened with open_file().'''
+        os.close(fd)
+
+    @staticmethod
+    def get_pid():
+        '''Return the PID of the current process.'''
+        return os.getpid()
+
     #
     # List NetNS info
     #
     def _dump_one_ns(self, path, registry):
         item = nsinfmsg()
-        item['netnsid'] = 0xffffffff  # default netnsid "unknown"
+        item['netnsid'] = 0xFFFFFFFF  # default netnsid "unknown"
         nsfd = 0
         info = nsidmsg()
         msg = nsidmsg()
         try:
-            nsfd = os.open(path, os.O_RDONLY)
-            item['inode'] = os.fstat(nsfd).st_ino
+            (nsfd, inode) = self.open_file(path)
+            item['inode'] = inode
             #
             # if the inode is registered, skip it
             #
@@ -419,9 +441,7 @@ class RTNL_API(object):
             #
             msg['attrs'] = [('NETNSA_FD', nsfd)]
             try:
-                for info in self.nlm_request(msg,
-                                             RTM_GETNSID,
-                                             NLM_F_REQUEST):
+                for info in self.nlm_request(msg, RTM_GETNSID, NLM_F_REQUEST):
                     # response to nlm_request() is a list or a generator,
                     # that's why loop
                     item['netnsid'] = info.get_attr('NETNSA_NSID')
@@ -433,7 +453,7 @@ class RTNL_API(object):
             raise SkipInode()
         finally:
             if nsfd > 0:
-                os.close(nsfd)
+                self.close_file(nsfd)
         item['header']['type'] = RTM_NEWNETNS
         item['header']['target'] = self.target
         item['event'] = 'RTM_NEWNETNS'
@@ -461,6 +481,33 @@ class RTNL_API(object):
             except SkipInode:
                 pass
 
+    def get_netnsid(self, nsid=None, pid=None, fd=None, target_nsid=None):
+        '''Return a dict containing the result of a RTM_GETNSID query.
+        This loosely corresponds to the "ip netns list-id" command.
+        '''
+        msg = nsidmsg()
+
+        if nsid is not None:
+            msg['attrs'].append(('NETNSA_NSID', nsid))
+
+        if pid is not None:
+            msg['attrs'].append(('NETNSA_PID', pid))
+
+        if fd is not None:
+            msg['attrs'].append(('NETNSA_FD', fd))
+
+        if target_nsid is not None:
+            msg['attrs'].append(('NETNSA_TARGET_NSID', target_nsid))
+
+        response = self.nlm_request(msg, RTM_GETNSID, NLM_F_REQUEST)
+        for r in response:
+            return {
+                'nsid': r.get_attr('NETNSA_NSID'),
+                'current_nsid': r.get_attr('NETNSA_CURRENT_NSID'),
+            }
+
+        return None
+
     def get_netns_info(self, list_proc=False):
         '''
         A prototype method to list available netns and associated
@@ -510,6 +557,7 @@ class RTNL_API(object):
                     yield item
             except OSError:
                 pass
+
     # 8<---------------------------------------------------------------
 
     # 8<---------------------------------------------------------------
@@ -521,11 +569,13 @@ class RTNL_API(object):
         Get default routes
         '''
         # according to iproute2/ip/iproute.c:print_route()
-        return [x for x in self.get_routes(family, table=table)
-                if (x.get_attr('RTA_DST', None) is None and
-                    x['dst_len'] == 0)]
+        return [
+            x
+            for x in self.get_routes(family, table=table)
+            if (x.get_attr('RTA_DST', None) is None and x['dst_len'] == 0)
+        ]
 
-    def link_lookup(self, **kwarg):
+    def link_lookup(self, match=None, **kwarg):
         '''
         Lookup interface index (indeces) by first level NLA
         value.
@@ -539,7 +589,7 @@ class RTNL_API(object):
         Please note, that link_lookup() returns list, not one
         value.
         '''
-        if set(kwarg) in ({'index', }, {'ifname', }, {'index', 'ifname'}):
+        if set(kwarg) in ({'index'}, {'ifname'}, {'index', 'ifname'}):
             # shortcut for index and ifname
             try:
                 for link in self.link('get', **kwarg):
@@ -548,7 +598,10 @@ class RTNL_API(object):
                 return []
         else:
             # otherwise fallback to the userspace filter
-            return [link['index'] for link in self.get_links(match=kwarg)]
+            return [
+                link['index'] for link in self.get_links(match=match or kwarg)
+            ]
+
     # 8<---------------------------------------------------------------
 
     # 8<---------------------------------------------------------------
@@ -607,6 +660,7 @@ class RTNL_API(object):
             self.put(rule, msg_type=RTM_DELRULE, msg_flags=flags)
             ret.append(rule)
         return ret
+
     # 8<---------------------------------------------------------------
 
     # 8<---------------------------------------------------------------
@@ -624,29 +678,31 @@ class RTNL_API(object):
         Possible keywords are NLA names for the `protinfo_bridge` class,
         without the prefix and in lower letters.
         '''
+        if command == 'set':
+            linkkwarg = dict()
+            linkkwarg['index'] = kwarg.pop('index', 0)
+            linkkwarg['kind'] = 'bridge_slave'
+            for key in kwarg:
+                linkkwarg[key] = kwarg[key]
+            return self.link(command, **linkkwarg)
         if (command in ('dump', 'show')) and ('match' not in kwarg):
             match = kwarg
         else:
             match = kwarg.pop('match', None)
 
         flags_dump = NLM_F_REQUEST | NLM_F_DUMP
-        flags_req = NLM_F_REQUEST | NLM_F_ACK
-        commands = {'set': (RTM_SETLINK, flags_req),
-                    'dump': (RTM_GETLINK, flags_dump),
-                    'show': (RTM_GETLINK, flags_dump)}
+        commands = {
+            'dump': (RTM_GETLINK, flags_dump),
+            'show': (RTM_GETLINK, flags_dump),
+        }
         (command, msg_flags) = commands.get(command, command)
 
         msg = ifinfmsg()
-        if command == RTM_GETLINK:
-            msg['index'] = kwarg.get('index', 0)
-        else:
-            msg['index'] = kwarg.pop('index', 0)
+        msg['index'] = kwarg.get('index', 0)
         msg['family'] = AF_BRIDGE
         protinfo = IPBrPortRequest(kwarg)
-        msg['attrs'].append(('IFLA_PROTINFO', protinfo, 0x8000))
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=msg_flags)
+        msg['attrs'].append(('IFLA_PROTINFO', dict(protinfo), 0x8000))
+        ret = self.nlm_request(msg, msg_type=command, msg_flags=msg_flags)
         if match is not None:
             ret = self._match(match, ret)
 
@@ -783,8 +839,10 @@ class RTNL_API(object):
 
         '''
         flags_req = NLM_F_REQUEST | NLM_F_ACK
-        commands = {'add': (RTM_SETLINK, flags_req),
-                    'del': (RTM_DELLINK, flags_req)}
+        commands = {
+            'add': (RTM_SETLINK, flags_req),
+            'del': (RTM_DELLINK, flags_req),
+        }
 
         kwarg['family'] = AF_BRIDGE
         kwarg['kwarg_filter'] = IPBridgeRequest
@@ -865,17 +923,20 @@ class RTNL_API(object):
         # nud -> state
         if 'nud' in kwarg:
             kwarg['state'] = kwarg.pop('nud')
-        if (command in ('add', 'del', 'append')) and \
-                not (kwarg.get('state', 0) & ndmsg.states['noarp']):
+        if (command in ('add', 'del', 'append')) and not (
+            kwarg.get('state', 0) & ndmsg.states['noarp']
+        ):
             # state must contain noarp in add / del / append
             kwarg['state'] = kwarg.pop('state', 0) | ndmsg.states['noarp']
             # other assumptions
-            if not kwarg.get('state', 0) & (ndmsg.states['permanent'] |
-                                            ndmsg.states['reachable']):
+            if not kwarg.get('state', 0) & (
+                ndmsg.states['permanent'] | ndmsg.states['reachable']
+            ):
                 # permanent (default) or reachable
                 kwarg['state'] |= ndmsg.states['permanent']
-            if not kwarg.get('flags', 0) & (ndmsg.flags['self'] |
-                                            ndmsg.flags['master']):
+            if not kwarg.get('flags', 0) & (
+                ndmsg.flags['self'] | ndmsg.flags['master']
+            ):
                 # self (default) or master
                 kwarg['flags'] = kwarg.get('flags', 0) | ndmsg.flags['self']
         #
@@ -951,23 +1012,42 @@ class RTNL_API(object):
         flags_change = flags_base | NLM_F_REPLACE
         flags_replace = flags_change | NLM_F_CREATE
 
-        commands = {'add': (RTM_NEWNEIGH, flags_make),
-                    'set': (RTM_NEWNEIGH, flags_replace),
-                    'replace': (RTM_NEWNEIGH, flags_replace),
-                    'change': (RTM_NEWNEIGH, flags_change),
-                    'del': (RTM_DELNEIGH, flags_make),
-                    'remove': (RTM_DELNEIGH, flags_make),
-                    'delete': (RTM_DELNEIGH, flags_make),
-                    'dump': (RTM_GETNEIGH, flags_dump),
-                    'get': (RTM_GETNEIGH, flags_base),
-                    'append': (RTM_NEWNEIGH, flags_append)}
-
-        (command, flags) = commands.get(command, command)
+        msg = ndmsg.ndmsg()
+        # fix kwarg
+        # FIXME: move to req?
         if 'nud' in kwarg:
             kwarg['state'] = kwarg.pop('nud')
-        msg = ndmsg.ndmsg()
+        if 'state' not in kwarg:
+            kwarg['state'] = 'permanent'
+        if 'family' not in kwarg and 'dst' in kwarg:
+            if '.' in kwarg['dst']:
+                kwarg['family'] = AF_INET
+            elif ':' in kwarg['dst']:
+                kwarg['family'] = AF_INET6
+
         for field in msg.fields:
+            if (
+                command == "dump"
+                and self.strict_check
+                and field[0] == "ifindex"
+            ):
+                continue
             msg[field[0]] = kwarg.pop(field[0], 0)
+
+        commands = {
+            'add': (RTM_NEWNEIGH, flags_make),
+            'set': (RTM_NEWNEIGH, flags_replace),
+            'replace': (RTM_NEWNEIGH, flags_replace),
+            'change': (RTM_NEWNEIGH, flags_change),
+            'del': (RTM_DELNEIGH, flags_make),
+            'remove': (RTM_DELNEIGH, flags_make),
+            'delete': (RTM_DELNEIGH, flags_make),
+            'dump': (RTM_GETNEIGH, flags_dump),
+            'get': (RTM_GETNEIGH, flags_base),
+            'append': (RTM_NEWNEIGH, flags_append),
+        }
+
+        (command, flags) = commands.get(command, command)
         msg['family'] = msg['family'] or AF_INET
         msg['attrs'] = []
         # fix nud kwarg
@@ -979,9 +1059,7 @@ class RTNL_API(object):
             if kwarg[key] is not None:
                 msg['attrs'].append([nla, kwarg[key]])
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=flags)
+        ret = self.nlm_request(msg, msg_type=command, msg_flags=flags)
         if match:
             ret = self._match(match, ret)
 
@@ -1253,6 +1331,20 @@ class RTNL_API(object):
             # bring link up
             ip.link("set", index=x, state="up")
 
+        Seting bridge or tunnel attributes require `kind` to be
+        specified in order to properly encode `IFLA_LINKINFO`::
+
+            ip.link("set",
+                    index=x,
+                    kind="bridge",
+                    br_forward_delay=2000)
+
+            ip.link("set",
+                    index=x,
+                    kind="gre",
+                    gre_local="10.0.0.1",
+                    gre_remote="10.1.0.103")
+
         Keyword "state" is reserved. State can be "up" or "down",
         it is a shortcut::
 
@@ -1322,16 +1414,18 @@ class RTNL_API(object):
         flags_req = NLM_F_REQUEST | NLM_F_ACK
         flags_create = flags_req | NLM_F_CREATE | NLM_F_EXCL
         flag_append = flags_create | NLM_F_APPEND
-        commands = {'set': (RTM_NEWLINK, flags_req),
-                    'update': (RTM_SETLINK, flags_create),
-                    'add': (RTM_NEWLINK, flags_create),
-                    'del': (RTM_DELLINK, flags_create),
-                    'property_add': (RTM_NEWLINKPROP, flag_append),
-                    'property_del': (RTM_DELLINKPROP, flags_req),
-                    'remove': (RTM_DELLINK, flags_create),
-                    'delete': (RTM_DELLINK, flags_create),
-                    'dump': (RTM_GETLINK, flags_dump),
-                    'get': (RTM_GETLINK, NLM_F_REQUEST)}
+        commands = {
+            'set': (RTM_NEWLINK, flags_req),
+            'update': (RTM_SETLINK, flags_create),
+            'add': (RTM_NEWLINK, flags_create),
+            'del': (RTM_DELLINK, flags_create),
+            'property_add': (RTM_NEWLINKPROP, flag_append),
+            'property_del': (RTM_DELLINKPROP, flags_req),
+            'remove': (RTM_DELLINK, flags_create),
+            'delete': (RTM_DELLINK, flags_create),
+            'dump': (RTM_GETLINK, flags_dump),
+            'get': (RTM_GETLINK, NLM_F_REQUEST),
+        }
 
         msg = ifinfmsg()
         # ifinfmsg fields
@@ -1354,9 +1448,9 @@ class RTNL_API(object):
 
         # UP/DOWN shortcut
         if 'state' in kwarg:
-            mask = 1                  # IFF_UP mask
+            mask = 1  # IFF_UP mask
             if kwarg['state'].lower() == 'up':
-                flags = 1             # 0 (down) or 1 (up)
+                flags = 1  # 0 (down) or 1 (up)
             del kwarg['state']
 
         # arp on/off shortcut
@@ -1374,10 +1468,12 @@ class RTNL_API(object):
                 if not isinstance(altname, (list, tuple, set)):
                     altname = [altname]
 
-                kwarg["IFLA_PROP_LIST"] = {"attrs": [
-                    ("IFLA_ALT_IFNAME", alt_ifname)
-                    for alt_ifname in altname
-                ]}
+                kwarg["IFLA_PROP_LIST"] = {
+                    "attrs": [
+                        ("IFLA_ALT_IFNAME", alt_ifname)
+                        for alt_ifname in altname
+                    ]
+                }
             else:
                 kwarg["IFLA_ALT_IFNAME"] = altname
 
@@ -1390,9 +1486,7 @@ class RTNL_API(object):
             if kwarg[key] is not None:
                 msg['attrs'].append([nla, kwarg[key]])
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=msg_flags)
+        ret = self.nlm_request(msg, msg_type=command, msg_flags=msg_flags)
         if match is not None:
             ret = self._match(match, ret)
 
@@ -1401,8 +1495,17 @@ class RTNL_API(object):
 
         return ret
 
-    def addr(self, command, index=None, address=None, mask=None,
-             family=None, scope=None, match=None, **kwarg):
+    def addr(
+        self,
+        command,
+        index=None,
+        address=None,
+        mask=None,
+        family=None,
+        scope=None,
+        match=None,
+        **kwarg
+    ):
         '''
         Address operations
 
@@ -1458,12 +1561,14 @@ class RTNL_API(object):
         flags_base = NLM_F_REQUEST | NLM_F_ACK
         flags_create = flags_base | NLM_F_CREATE | NLM_F_EXCL
         flags_replace = flags_base | NLM_F_REPLACE | NLM_F_CREATE
-        commands = {'add': (RTM_NEWADDR, flags_create),
-                    'del': (RTM_DELADDR, flags_create),
-                    'remove': (RTM_DELADDR, flags_create),
-                    'delete': (RTM_DELADDR, flags_create),
-                    'replace': (RTM_NEWADDR, flags_replace),
-                    'dump': (RTM_GETADDR, flags_dump)}
+        commands = {
+            'add': (RTM_NEWADDR, flags_create),
+            'del': (RTM_DELADDR, flags_create),
+            'remove': (RTM_DELADDR, flags_create),
+            'delete': (RTM_DELADDR, flags_create),
+            'replace': (RTM_NEWADDR, flags_replace),
+            'dump': (RTM_GETADDR, flags_dump),
+        }
         (command, flags) = commands.get(command, command)
 
         # fetch args
@@ -1500,9 +1605,11 @@ class RTNL_API(object):
             pass
 
         # inject IFA_LOCAL, if family is AF_INET and IFA_LOCAL is not set
-        if family == AF_INET and \
-                kwarg.get('address') and \
-                kwarg.get('local') is None:
+        if (
+            family == AF_INET
+            and kwarg.get('address')
+            and kwarg.get('local') is None
+        ):
             kwarg['local'] = kwarg['address']
 
         # patch broadcast, if needed
@@ -1515,11 +1622,12 @@ class RTNL_API(object):
             if kwarg[key] not in (None, ''):
                 msg['attrs'].append([nla, kwarg[key]])
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=flags,
-                               terminate=lambda x: x['header']['type'] ==
-                               NLMSG_ERROR)
+        ret = self.nlm_request(
+            msg,
+            msg_type=command,
+            msg_flags=flags,
+            terminate=lambda x: x['header']['type'] == NLMSG_ERROR,
+        )
         if match:
             ret = self._match(match, ret)
 
@@ -1607,26 +1715,29 @@ class RTNL_API(object):
         flags_change = flags_base | NLM_F_REPLACE
         flags_replace = flags_change | NLM_F_CREATE
 
-        commands = {'add': (RTM_NEWQDISC, flags_make),
-                    'del': (RTM_DELQDISC, flags_make),
-                    'remove': (RTM_DELQDISC, flags_make),
-                    'delete': (RTM_DELQDISC, flags_make),
-                    'change': (RTM_NEWQDISC, flags_change),
-                    'replace': (RTM_NEWQDISC, flags_replace),
-                    'add-class': (RTM_NEWTCLASS, flags_make),
-                    'del-class': (RTM_DELTCLASS, flags_make),
-                    'change-class': (RTM_NEWTCLASS, flags_change),
-                    'replace-class': (RTM_NEWTCLASS, flags_replace),
-                    'add-filter': (RTM_NEWTFILTER, flags_make),
-                    'del-filter': (RTM_DELTFILTER, flags_make),
-                    'change-filter': (RTM_NEWTFILTER, flags_change),
-                    'replace-filter': (RTM_NEWTFILTER, flags_replace)}
+        commands = {
+            'add': (RTM_NEWQDISC, flags_make),
+            'del': (RTM_DELQDISC, flags_make),
+            'remove': (RTM_DELQDISC, flags_make),
+            'delete': (RTM_DELQDISC, flags_make),
+            'change': (RTM_NEWQDISC, flags_change),
+            'replace': (RTM_NEWQDISC, flags_replace),
+            'add-class': (RTM_NEWTCLASS, flags_make),
+            'del-class': (RTM_DELTCLASS, flags_make),
+            'change-class': (RTM_NEWTCLASS, flags_change),
+            'replace-class': (RTM_NEWTCLASS, flags_replace),
+            'add-filter': (RTM_NEWTFILTER, flags_make),
+            'del-filter': (RTM_DELTFILTER, flags_make),
+            'change-filter': (RTM_NEWTFILTER, flags_change),
+            'replace-filter': (RTM_NEWTFILTER, flags_replace),
+        }
         if isinstance(command, int):
             command = (command, flags_make)
         if command == 'del':
             if index == 0:
-                index = [x['index'] for x in self.get_links()
-                         if x['index'] != 1]
+                index = [
+                    x['index'] for x in self.get_links() if x['index'] != 1
+                ]
             if isinstance(index, (list, tuple, set)):
                 return list(chain(*(self.tc('del', index=x) for x in index)))
         command, flags = commands.get(command, command)
@@ -1839,6 +1950,20 @@ class RTNL_API(object):
                             'action': 'End.DT6',
                             'table':'10'})
 
+        Create SEG6LOCAL tunnel End.DT4 action (kernel >= 5.11)::
+            # $ sudo modprobe vrf
+            # $ sudo sysctl -w net.vrf.strict_mode=1
+            ip.link('add',
+                    ifname='vrf-foo',
+                    kind='vrf',
+                    vrf_table=10)
+            ip.route('add',
+                     dst='2001:0:0:10::2/128',
+                     oif=idx,
+                     encap={'type': 'seg6local',
+                            'action': 'End.DT4',
+                            'vrf_table': 10})
+
         Create SEG6LOCAL tunnel End.B6 action (kernel >= 4.14)::
 
             ip.route('add',
@@ -1899,25 +2024,29 @@ class RTNL_API(object):
             match = kwarg.pop('match', None)
         callback = kwarg.pop('callback', None)
 
-        commands = {'add': (RTM_NEWROUTE, flags_make),
-                    'set': (RTM_NEWROUTE, flags_replace),
-                    'replace': (RTM_NEWROUTE, flags_replace),
-                    'change': (RTM_NEWROUTE, flags_change),
-                    'append': (RTM_NEWROUTE, flags_append),
-                    'del': (RTM_DELROUTE, flags_make),
-                    'remove': (RTM_DELROUTE, flags_make),
-                    'delete': (RTM_DELROUTE, flags_make),
-                    'get': (RTM_GETROUTE, NLM_F_REQUEST),
-                    'show': (RTM_GETROUTE, flags_dump),
-                    'dump': (RTM_GETROUTE, flags_dump)}
+        commands = {
+            'add': (RTM_NEWROUTE, flags_make),
+            'set': (RTM_NEWROUTE, flags_replace),
+            'replace': (RTM_NEWROUTE, flags_replace),
+            'change': (RTM_NEWROUTE, flags_change),
+            'append': (RTM_NEWROUTE, flags_append),
+            'del': (RTM_DELROUTE, flags_make),
+            'remove': (RTM_DELROUTE, flags_make),
+            'delete': (RTM_DELROUTE, flags_make),
+            'get': (RTM_GETROUTE, NLM_F_REQUEST),
+            'show': (RTM_GETROUTE, flags_dump),
+            'dump': (RTM_GETROUTE, flags_dump),
+        }
         (command, flags) = commands.get(command, command)
         msg = rtmsg()
 
-        # table is mandatory; by default == 254
+        # table is mandatory without strict_check; by default == 254
         # if table is not defined in kwarg, save it there
-        # also for nla_attr:
-        table = kwarg.get('table', 254)
-        msg['table'] = table if table <= 255 else 252
+        # also for nla_attr. Do not set it in strict_check, use
+        # NLA instead
+        if not self.strict_check:
+            table = kwarg.get('table', 254)
+            msg['table'] = table if table <= 255 else 252
         msg['family'] = kwarg.pop('family', AF_INET)
         msg['scope'] = kwarg.pop('scope', rt_scope['universe'])
         msg['dst_len'] = kwarg.pop('dst_len', None) or kwarg.pop('mask', 0)
@@ -1941,23 +2070,31 @@ class RTNL_API(object):
                 msg['attrs'].append([nla, kwarg[key]])
                 # fix IP family, if needed
                 if msg['family'] in (AF_UNSPEC, 255):
-                    if key in ('dst', 'src', 'gateway', 'prefsrc', 'newdst') \
-                            and isinstance(kwarg[key], basestring):
-                        msg['family'] = AF_INET6 if kwarg[key].find(':') >= 0 \
-                            else AF_INET
+                    if key in (
+                        'dst',
+                        'src',
+                        'gateway',
+                        'prefsrc',
+                        'newdst',
+                    ) and isinstance(kwarg[key], basestring):
+                        msg['family'] = (
+                            AF_INET6 if kwarg[key].find(':') >= 0 else AF_INET
+                        )
                     elif key == 'multipath' and len(kwarg[key]) > 0:
                         hop = kwarg[key][0]
                         attrs = hop.get('attrs', [])
                         for attr in attrs:
                             if attr[0] == 'RTA_GATEWAY':
-                                msg['family'] = AF_INET6 if \
-                                    attr[1].find(':') >= 0 else AF_INET
+                                msg['family'] = (
+                                    AF_INET6
+                                    if attr[1].find(':') >= 0
+                                    else AF_INET
+                                )
                                 break
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=flags,
-                               callback=callback)
+        ret = self.nlm_request(
+            msg, msg_type=command, msg_flags=flags, callback=callback
+        )
         if match:
             ret = self._match(match, ret)
 
@@ -1988,8 +2125,8 @@ class RTNL_API(object):
                 routing's rule
             - oifname — Output interface for Interface Based (Policy Based)
                 routing's rule
-            - uid_range — Range of user identifiers, as a string like "1000:1234"
-            - dport_range — Range of destination ports, as a string like "80-120"
+            - uid_range — Range of user identifiers, a string like "1000:1234"
+            - dport_range — Range of destination ports, a string like "80-120"
             - sport_range — Range of source ports, as a string like "80-120"
 
         All packets route via table 10::
@@ -2043,11 +2180,13 @@ class RTNL_API(object):
         flags_make = flags_base | NLM_F_CREATE | NLM_F_EXCL
         flags_dump = NLM_F_REQUEST | NLM_F_ROOT | NLM_F_ATOMIC
 
-        commands = {'add': (RTM_NEWRULE, flags_make),
-                    'del': (RTM_DELRULE, flags_make),
-                    'remove': (RTM_DELRULE, flags_make),
-                    'delete': (RTM_DELRULE, flags_make),
-                    'dump': (RTM_GETRULE, flags_dump)}
+        commands = {
+            'add': (RTM_NEWRULE, flags_make),
+            'del': (RTM_DELRULE, flags_make),
+            'remove': (RTM_DELRULE, flags_make),
+            'delete': (RTM_DELRULE, flags_make),
+            'dump': (RTM_GETRULE, flags_dump),
+        }
         if isinstance(command, int):
             command = (command, flags_make)
         command, flags = commands.get(command, command)
@@ -2055,32 +2194,38 @@ class RTNL_API(object):
         if argv:
             # this code block will be removed in some release
             log.error('rule(): positional parameters are deprecated')
-            names = ['table', 'priority', 'action', 'family',
-                     'src', 'src_len', 'dst', 'dst_len', 'fwmark',
-                     'iifname', 'oifname']
+            names = [
+                'table',
+                'priority',
+                'action',
+                'family',
+                'src',
+                'src_len',
+                'dst',
+                'dst_len',
+                'fwmark',
+                'iifname',
+                'oifname',
+            ]
             kwarg.update(dict(zip(names, argv)))
 
         kwarg = IPRuleRequest(kwarg)
         msg = fibmsg()
         table = kwarg.get('table', 0)
         msg['table'] = table if table <= 255 else 252
-        for key in ('family',
-                    'src_len',
-                    'dst_len',
-                    'action',
-                    'tos',
-                    'flags'):
+        for key in ('family', 'src_len', 'dst_len', 'action', 'tos', 'flags'):
             msg[key] = kwarg.pop(key, 0)
         msg['attrs'] = []
 
         for key in kwarg:
+            if command == RTM_GETRULE and self.strict_check:
+                if key in ("match", "priority"):
+                    continue
             nla = fibmsg.name2nla(key)
             if kwarg[key] is not None:
                 msg['attrs'].append([nla, kwarg[key]])
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=flags)
+        ret = self.nlm_request(msg, msg_type=command, msg_flags=flags)
 
         if 'match' in kwarg:
             ret = self._match(kwarg['match'], ret)
@@ -2099,17 +2244,17 @@ class RTNL_API(object):
         else:
             match = kwarg.pop('match', None)
 
-        commands = {'dump': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_DUMP),
-                    'get': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_ACK)}
+        commands = {
+            'dump': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_DUMP),
+            'get': (RTM_GETSTATS, NLM_F_REQUEST | NLM_F_ACK),
+        }
         command, flags = commands.get(command, command)
 
         msg = ifstatsmsg()
         msg['filter_mask'] = kwarg.get('filter_mask', 31)
         msg['ifindex'] = kwarg.get('ifindex', 0)
 
-        ret = self.nlm_request(msg,
-                               msg_type=command,
-                               msg_flags=flags)
+        ret = self.nlm_request(msg, msg_type=command, msg_flags=flags)
         if match is not None:
             ret = self._match(match, ret)
 
@@ -2117,6 +2262,7 @@ class RTNL_API(object):
             ret = tuple(ret)
 
         return ret
+
     # 8<---------------------------------------------------------------
 
 
@@ -2145,6 +2291,7 @@ class IPBatch(RTNL_API, IPBatchSocket):
         IPRoute().sendto(data, (0, 0))
 
     '''
+
     pass
 
 
@@ -2152,6 +2299,7 @@ class IPRoute(RTNL_API, IPRSocket):
     '''
     Regular ordinary utility class, see RTNL API for the list of methods.
     '''
+
     pass
 
 
@@ -2160,4 +2308,5 @@ class RawIPRoute(RTNL_API, RawIPRSocket):
     The same as `IPRoute`, but does not use the netlink proxy.
     Thus it can not manage e.g. tun/tap interfaces.
     '''
+
     pass
