@@ -5,6 +5,7 @@ class NetlinkError(Exception):
     '''
     Base netlink error
     '''
+
     def __init__(self, code, msg=None):
         msg = msg or os.strerror(code)
         super(NetlinkError, self).__init__(code, msg)
@@ -18,6 +19,7 @@ class NetlinkDecodeError(Exception):
 
     Incapsulates underlying error for the following analysis
     '''
+
     def __init__(self, exception):
         self.exception = exception
 
@@ -26,6 +28,7 @@ class NetlinkHeaderDecodeError(NetlinkDecodeError):
     '''
     The error occured while decoding a header
     '''
+
     pass
 
 
@@ -33,6 +36,7 @@ class NetlinkDataDecodeError(NetlinkDecodeError):
     '''
     The error occured while decoding the message fields
     '''
+
     pass
 
 
@@ -40,6 +44,7 @@ class NetlinkNLADecodeError(NetlinkDecodeError):
     '''
     The error occured while decoding NLA chain
     '''
+
     pass
 
 
@@ -49,8 +54,18 @@ class IPSetError(NetlinkError):
 
     Messages are imported from errcode.c
     '''
+
     pass
 
 
+class NetlinkDumpInterrupted(NetlinkError):
+    '''
+    Raised when NLM_F_DUMP_INTR is set in the flags.
+    '''
+
+    def __init__(self):
+        super(NetlinkDumpInterrupted, self).__init__(-1, 'dump interrupted')
+
+
 class SkipInode(Exception):
     pass
